package cn.breeze.elleai.application.service;

import cn.breeze.elleai.application.dto.PageResult;
import cn.breeze.elleai.application.dto.inner.AiChatComplateResultDto;
import cn.breeze.elleai.application.dto.request.*;
import cn.breeze.elleai.application.dto.response.*;
import cn.breeze.elleai.domain.sparring.model.request.*;
import cn.breeze.elleai.domain.sparring.model.response.*;
import cn.breeze.elleai.domain.sparring.service.ChatCompletionService;
import cn.breeze.elleai.domain.sparring.service.KbService;
import cn.breeze.elleai.domain.sparring.service.KbTagService;
import cn.breeze.elleai.facade.TencentCloudFacade;
import cn.breeze.elleai.util.UserPrincipal;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.exceptions.ExceptionUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.mybatisflex.core.paginate.Page;
import com.tencentcloudapi.common.exception.TencentCloudSDKException;
import jakarta.annotation.PostConstruct;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.*;
import org.springframework.http.codec.ServerSentEvent;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Flux;

import java.util.*;
import java.util.stream.Collectors;


/**
 * 应用服务
 */
@Slf4j
@Component
@RequiredArgsConstructor
public class AppChatCompletionService {

    @Value("${dify.chat_api_base}")
    private String difyBase;

    @Value("${dify.chat_api_key}")
    private String apiKey;

    @Value("${deepseek.api_base}")
    private String dsApiBase;

    @Value("${deepseek.app_key}")
    private String dsAppKey;

    @Value("${deepseek.secret_id}")
    private String dsSecretId;

    @Value("${deepseek.app_id}")
    private String dsAppId;

    @Value("${deepseek.diss_api_base}")
    private String dissApiBase;

    private final RestTemplate restTemplate = new RestTemplate();

    private final ChatCompletionService chatCompletionService;

    private final TencentCloudFacade tencentCloudFacade;

    private final KbService kbService;

    private final KbTagService kbTagService;

    private WebClient webClient;


    @PostConstruct
    public void init() {
        webClient = WebClient.builder().baseUrl(difyBase)
                .defaultHeader(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_JSON_VALUE)
                .defaultHeader(HttpHeaders.AUTHORIZATION, "Bearer "+apiKey)
                .defaultHeader(HttpHeaders.CACHE_CONTROL, "no-cache")
                .build();
    }

    /************************************************** AI助手 **************************************************/

    /**
     * 更新助手
     * @param id
     * @param status
     */
    public void updateQaAssistantStatus(Integer id, Integer status) {
        chatCompletionService.updateQaAssistantStatus(id, status);
    }

    /**
     * 删除助手
     * @param id
     */
    public void deleteQaAssistant(Integer id) {
        chatCompletionService.deleteQaAssistant(id);
    }

    /**
     * 获取助手详情
     * @param id
     * @return
     */
    public QaAssistantDto qaAssistantDetail(Integer id) {
        QaAssistantResponseModel model = chatCompletionService.qaAssistantDetail(id);

        QaAssistantDto result = BeanUtil.copyProperties(model, QaAssistantDto.class);
        if(StrUtil.isNotEmpty(model.getVisibleRoleIds())) {
            result.setVisibleRoleList(JSONObject.parseArray(model.getVisibleRoleIds(), String.class));
        }
        if(StrUtil.isNotEmpty(model.getCategoryIds())) {
            result.setCategoryIdList(JSONObject.parseArray(model.getCategoryIds(), Integer.class));
            if(CollectionUtil.isNotEmpty(result.getCategoryIdList())) {
                List<KbTagResponseModel> tagList = kbTagService.kbTagList(result.getCategoryIdList());
                List<String> tagNameList = tagList.stream().map(KbTagResponseModel::getName).collect(Collectors.toList());
                result.setCategoryNameList(tagNameList);
            }
        }

        return result;
    }


    /**
     * 保存助手
     * @param dto
     */
    public void saveQaAssistant(QaAssistantSaveDto dto) {

        QaAssistantSaveModel model = BeanUtil.copyProperties(dto, QaAssistantSaveModel.class);
        model.setCategoryIds(JSONObject.toJSONString(dto.getCategoryIdList()));
        model.setVisibleRoleIds(JSONObject.toJSONString(dto.getVisibleRoleList()));
        chatCompletionService.saveQaAssistant(model);
    }


    /**
     * 助手分页查询
     * @param request
     * @return
     */
    public PageResult<QaAssistantDto> qaAssistantPaginQuery(QaAssistantRequestDto request) {

        QaAssistantRequestModel requestModel = BeanUtil.copyProperties(request, QaAssistantRequestModel.class);

        Page<QaAssistantResponseModel> page = chatCompletionService.qaAssistantPaginQuery(requestModel);

        PageResult<QaAssistantDto> pageResult = PageResult.of(request.getPageNo(), request.getPageSize(), (int) page.getTotalRow(), null);

        if(CollectionUtil.isNotEmpty(page.getRecords())) {
            List<QaAssistantDto> dtoList = page.getRecords().stream().map(v -> {
                QaAssistantDto dto = BeanUtil.copyProperties(v, QaAssistantDto.class);

                if(StrUtil.isNotEmpty(v.getCategoryIds())) {
                    dto.setCategoryIdList(JSONObject.parseArray(v.getCategoryIds(), Integer.class));
                    if(CollectionUtil.isNotEmpty(dto.getCategoryIdList())) {
                        List<KbTagResponseModel> tagList = kbTagService.kbTagList(dto.getCategoryIdList());
                        List<String> tagNameList = tagList.stream().map(KbTagResponseModel::getName).collect(Collectors.toList());
                        dto.setCategoryNameList(tagNameList);
                    }
                }
                if(StrUtil.isNotEmpty(v.getVisibleRoleIds())) {
                    dto.setVisibleRoleList(JSONObject.parseArray(v.getVisibleRoleIds(), String.class));
                }
                return dto;
            }).collect(Collectors.toList());

            pageResult.setItems(dtoList);
        }
        return pageResult;
    }


    /**
     * 助手列表查询
     * @param request
     * @return
     */
    public List<QaAssistantDto> qaAssistantList(QaAssistantRequestDto request) {

        List<QaAssistantDto> resultList = new ArrayList<>();

        QaAssistantRequestModel requestModel = BeanUtil.copyProperties(request, QaAssistantRequestModel.class);

        List<QaAssistantResponseModel> modelList = chatCompletionService.qaAssistantList(requestModel);

        if(CollectionUtil.isNotEmpty(modelList)) {
            resultList = modelList.stream().map(v -> {
                QaAssistantDto dto = BeanUtil.copyProperties(v, QaAssistantDto.class);

                if(StrUtil.isNotEmpty(v.getCategoryIds())) {
                    dto.setCategoryIdList(JSONObject.parseArray(v.getCategoryIds(), Integer.class));
                }
                if(StrUtil.isNotEmpty(v.getVisibleRoleIds())) {
                    dto.setVisibleRoleList(JSONObject.parseArray(v.getVisibleRoleIds(), String.class));
                }
                return dto;
            }).collect(Collectors.toList());
        }
        return resultList;
    }


    /************************************************** AI问答会话 **************************************************/

    /**
     * 获取AI问答会话详情
     * @param sessionId
     * @return
     */
    public List<UserChatCompletionHistoryDto> userQaDetail(Integer sessionId) {
        List<UserChatCompletionHistoryDto> resultList = new ArrayList<>();

        List<UserChatCompletionHistoryResponseModel> models = chatCompletionService.userQaHistoryDetail(sessionId);
        if(CollectionUtil.isNotEmpty(models)) {
            resultList = models.stream().map(v -> {
                UserChatCompletionHistoryDto dto = BeanUtil.copyProperties(v, UserChatCompletionHistoryDto.class);
                return dto;
            }).collect(Collectors.toList());
        }

        return resultList;
    }


    /**
     * AI问答会话分页查询
     * @param request
     * @return
     */
    public PageResult<UserChatCompletionDto> userQaPaginQuery(UserQaRequestDto request) {

        UserQaRequestModel requestModel = BeanUtil.copyProperties(request, UserQaRequestModel.class);

        Page<UserChatCompletionResponseModel> page = chatCompletionService.userQaPaginQuery(requestModel);

        PageResult<UserChatCompletionDto> pageResult = PageResult.of(request.getPageNo(), request.getPageSize(), (int) page.getTotalRow(), null);
        if(CollectionUtil.isNotEmpty(page.getRecords())) {
            List<UserChatCompletionDto> dtoList = page.getRecords().stream().map(v -> BeanUtil.copyProperties(v, UserChatCompletionDto.class)).collect(Collectors.toList());
            pageResult.setItems(dtoList);
        }
        return pageResult;
    }


    /**
     * AI问答会话列表查询
     * @param request
     * @return
     */
    public PageResult<UserChatCompletionDto> userQaMobilePaginQuery(UserQaRequestDto request) {

        UserQaRequestModel requestModel = BeanUtil.copyProperties(request, UserQaRequestModel.class);

        Page<UserChatCompletionResponseModel> page = chatCompletionService.userQaPaginQuery(requestModel);

        PageResult<UserChatCompletionDto> pageResult = PageResult.of(request.getPageNo(), request.getPageSize(), (int) page.getTotalRow(), null);

        if(CollectionUtil.isNotEmpty(page.getRecords())) {
            List<UserChatCompletionDto> dtoList = page.getRecords().stream().map(v -> {
                UserChatCompletionDto dto = BeanUtil.copyProperties(v, UserChatCompletionDto.class);
                return dto;
            }).collect(Collectors.toList());

            pageResult.setItems(dtoList);
        }
        return pageResult;
    }

    /**
     * AI问答会话分页查询
     * @param request
     * @return
     */
    public PageResult<UserChatCompletionMobileDto> userQaHistoryPaginQuery(UserPrincipal userPrincipal, UserQaHistoryRequestDto request) {

        UserQaRequestDto req = new UserQaRequestDto();
        req.setUserId(userPrincipal.getUserId());
        req.setPageNo(request.getPageNo());
        req.setPageSize(request.getPageSize());

        PageResult<UserChatCompletionDto> userChatCompletionPageResult = userQaMobilePaginQuery(req);

        PageResult<UserChatCompletionMobileDto> pageResult = PageResult.of(userChatCompletionPageResult.getPageNo(), userChatCompletionPageResult.getPageSize(), userChatCompletionPageResult.getTotal(), null);

        if(Objects.nonNull(userChatCompletionPageResult) && CollectionUtil.isNotEmpty(userChatCompletionPageResult.getItems())) {

            List<UserChatCompletionMobileDto> items = userChatCompletionPageResult.getItems().stream().map(v -> {
                UserChatCompletionMobileDto dto = BeanUtil.copyProperties(v, UserChatCompletionMobileDto.class);
                dto.setContent(v.getFirstQuestion());
                return dto;
            }).collect(Collectors.toList());

            BeanUtil.copyProperties(userChatCompletionPageResult, pageResult);
            pageResult.setItems(items);
        }

        return pageResult;
    }

    /**
     * 移动端历史会话明细
     * @param userPrincipal
     * @param chatCompletionId
     * @param pageNo
     * @param pageSize
     * @return
     */
    public PageResult<UserChatCompletionHistoryMobileDto> userQaHistoryMobilePaginQuery(UserPrincipal userPrincipal, Integer chatCompletionId, Integer pageNo, Integer pageSize) {

        pageNo = Objects.isNull(pageNo) ? 1 : pageNo;
        pageSize = Objects.isNull(pageSize) ? 10 : pageSize;

        UserQaRequestDto req = new UserQaRequestDto();
        req.setUserId(userPrincipal.getUserId());

        Page<UserChatCompletionHistoryResponseModel> historyDetailPageResult = chatCompletionService.userQaHistoryDetailPaginQuery(chatCompletionId, pageNo, pageSize);

        PageResult<UserChatCompletionHistoryMobileDto> pageResult = PageResult.of(pageNo, pageSize, (int) historyDetailPageResult.getTotalRow(), null);

        if(Objects.nonNull(historyDetailPageResult) && CollectionUtil.isNotEmpty(historyDetailPageResult.getRecords())) {

            List<UserChatCompletionHistoryMobileDto> items = historyDetailPageResult.getRecords().stream().map(v -> {
                UserChatCompletionHistoryMobileDto dto = BeanUtil.copyProperties(v, UserChatCompletionHistoryMobileDto.class);
                return dto;
            }).collect(Collectors.toList());
            pageResult.setItems(items);
        }

        return pageResult;
    }

    /**
     * 用户提问
     * @param request
     * @return
     */
    public UserAskResultMobileDto userAsk(UserPrincipal userPrincipal, UserQaMobileRequestDto request) {
        UserAskResultMobileDto result = new UserAskResultMobileDto();

        // 获取会话详情
        UserQaRequestModel requestModel = new UserQaRequestModel();
        requestModel.setChatCompletionId(request.getChatCompletionId());
        requestModel.setUserId(userPrincipal.getUserId());
        UserChatCompletionResponseModel ccModel = chatCompletionService.userQaDetail(requestModel);

        String sessionId = "";
        if(Objects.nonNull(ccModel) && StrUtil.isNotEmpty(ccModel.getSessionId())) {
            sessionId = ccModel.getSessionId();
        }

        // 更新会话信息
        UserChatCompletionSaveModel saveModel = new UserChatCompletionSaveModel();
        saveModel.setUserId(userPrincipal.getUserId());
        saveModel.setUserName(userPrincipal.getUserName());
        saveModel.setId(request.getChatCompletionId());
        saveModel.setLastQuestion(request.getContent());
        saveModel.setShopId(userPrincipal.getShopId());
        saveModel.setShopName(userPrincipal.getShopName());

        // 新会话首次提问
        if(Objects.isNull(request.getChatCompletionId())) {
            saveModel.setCreateTime(new Date());
            saveModel.setFirstQuestion(request.getContent());
        }
        Integer chatCompletionId = chatCompletionService.saveUserQaSession(saveModel);

        // 保存问答详情
        Integer userQaRecordId = chatCompletionService.saveUserQaRecord(chatCompletionId, 0, null, request.getContent(), request.getAssistantId(), "");

        // 问一下AI
        AiChatComplateResultDto complateResultDto = ask4Knowledge(sessionId, userPrincipal.getUserId(), request.getContent(), request.getAssistantId());
        if(Objects.nonNull(complateResultDto)) {
            String replyContent = complateResultDto.getContent();

            // 更新DIFY会话ID
            if(StrUtil.isEmpty(sessionId)) {
                UserChatCompletionSaveModel updateModel = new UserChatCompletionSaveModel();
                updateModel.setId(chatCompletionId);
                updateModel.setSessionId(sessionId);
                chatCompletionService.saveUserQaSession(updateModel);
            }

            // 保存AI问答详情
            chatCompletionService.saveUserQaRecord(chatCompletionId, 1, userQaRecordId, replyContent, request.getAssistantId(), "");

            result.setChatCompletionId(chatCompletionId);
            result.setReplyContent(replyContent);
            result.setMessageId(userQaRecordId);
        }
        return result;
    }


    /**
     * 用户提问
     * @param request
     * @return
     */
    public Flux<ServerSentEvent<UserAskResultMobileDto>> userAsk4Stream(UserPrincipal userPrincipal, UserQaMobileRequestDto request) {
        // 获取会话详情
        UserQaRequestModel requestModel = new UserQaRequestModel();
        requestModel.setChatCompletionId(request.getChatCompletionId());
        requestModel.setUserId(userPrincipal.getUserId());
        UserChatCompletionResponseModel ccModel = chatCompletionService.userQaDetail(requestModel);

        String sessionId = "";
        if(Objects.nonNull(ccModel) && StrUtil.isNotEmpty(ccModel.getSessionId())) {
            sessionId = ccModel.getSessionId();
        }

        // 更新会话信息
        UserChatCompletionSaveModel saveModel = new UserChatCompletionSaveModel();
        saveModel.setUserId(userPrincipal.getUserId());
        saveModel.setUserName(userPrincipal.getUserName());
        saveModel.setId(request.getChatCompletionId());
        saveModel.setLastQuestion(request.getContent());
        saveModel.setShopId(userPrincipal.getShopId());
        saveModel.setShopName(userPrincipal.getShopName());

        // 新会话首次提问
        if(Objects.isNull(request.getChatCompletionId())) {
            saveModel.setCreateTime(new Date());
            saveModel.setFirstQuestion(request.getContent());
        }
        Integer chatCompletionId = chatCompletionService.saveUserQaSession(saveModel);

        // 保存问答详情
        Integer userQaRecordId = chatCompletionService.saveUserQaRecord(chatCompletionId, 0, null, request.getContent(), request.getAssistantId(), "");

        // 获取随机问题
        final List<HotQaMobileDto> hots = Lists.newArrayList();
        PageResult<HotQaMobileDto> pageResult = this.hotQaList(new QaAssistantRequestDto());
        if(Objects.nonNull(pageResult) && CollectionUtil.isNotEmpty(pageResult.getItems())) {
            List<HotQaMobileDto> items = pageResult.getItems();
            if(items.size() > 2){
                Collections.shuffle(items);
                items = items.subList(0, 2);
            }
            hots.addAll(items);
        }
        List<HotQaResponseModel> hotQaList = chatCompletionService.hotQaList(new HotQaRequestModel());
        if(CollectionUtil.isNotEmpty(hotQaList)) {
            Collections.shuffle(hotQaList);
            if(hotQaList.size() > 2) {
                hotQaList = hotQaList.subList(0, 2);
            }
            hotQaList.forEach(v -> {
                HotQaMobileDto dto = new HotQaMobileDto();
                dto.setQuestion(v.getQuestion());
                hots.add(dto);
            });
        }

        Map<String, String> inputs = new HashMap<>();
        inputs.put("assistant_id", Objects.nonNull(request.getAssistantId()) ? String.valueOf(request.getAssistantId()) : "");
        JSONObject param = new JSONObject();
        param.put("query", request.getContent());
        param.put("inputs", inputs);
        param.put("response_mode", "streaming");
        param.put("conversation_id", sessionId);
        param.put("user", userPrincipal.getUserId());

        final StringBuffer buffer = new StringBuffer();
        final String[] difySessionId = {""};
        String finalSessionId = sessionId;
        return webClient.post().uri("/chat-messages").accept(MediaType.TEXT_EVENT_STREAM).bodyValue(param.toJSONString()).exchangeToFlux(r -> r.bodyToFlux(String.class))
                .mapNotNull(v -> {
                    JSONObject json = JSONObject.parseObject(v);
                    String answer = json.getString("answer");
                    if (ObjectUtil.isNotNull(answer)) {
                        buffer.append(answer);
                    }
                    if(ObjectUtil.isEmpty(difySessionId[0]) && ObjectUtil.isNotEmpty(json.getString("conversation_id"))) {
                        difySessionId[0] = json.getString("conversation_id");
                    }

                    UserAskResultMobileDto result = new UserAskResultMobileDto();
                    result.setReplyContent(buffer.toString());
                    result.setChatCompletionId(chatCompletionId);
                    result.setMessageId(userQaRecordId);
                    result.setHots(hots);
                    return result;
                }).doOnComplete(
                        () -> {
                            String replyContent = buffer.toString();
                            // 更新DIFY会话ID
                            if(StrUtil.isEmpty(finalSessionId)) {
                                UserChatCompletionSaveModel updateModel = new UserChatCompletionSaveModel();
                                updateModel.setId(chatCompletionId);
                                updateModel.setSessionId(difySessionId[0]);
                                chatCompletionService.saveUserQaSession(updateModel);
                            }
                            // 保存AI问答详情
                            chatCompletionService.saveUserQaRecord(chatCompletionId, 1, userQaRecordId, replyContent, request.getAssistantId(), "");
                        }
                ).map(v -> ServerSentEvent.builder(v).build());
    }

    /**
     * 用户提问
     * @param request
     * @return
     */
    public Flux<ServerSentEvent<UserAskResultMobileDto>> userAsk4Stream2(UserPrincipal userPrincipal, UserQaMobileRequestDto request) {
        // 获取会话详情
        UserQaRequestModel requestModel = new UserQaRequestModel();
        requestModel.setChatCompletionId(request.getChatCompletionId());
        requestModel.setUserId(userPrincipal.getUserId());
        UserChatCompletionResponseModel ccModel = chatCompletionService.userQaDetail(requestModel);

        String sessionId = "";
        if(Objects.nonNull(ccModel) && StrUtil.isNotEmpty(ccModel.getSessionId())) {
            sessionId = ccModel.getSessionId();
        }
        if(StringUtils.isEmpty(sessionId)) {
            sessionId = UUID.randomUUID().toString();
        }

        // 更新会话信息
        UserChatCompletionSaveModel saveModel = new UserChatCompletionSaveModel();
        saveModel.setUserId(userPrincipal.getUserId());
        saveModel.setUserName(userPrincipal.getUserName());
        saveModel.setId(request.getChatCompletionId());
        saveModel.setLastQuestion(request.getContent());
        saveModel.setShopId(userPrincipal.getShopId());
        saveModel.setShopName(userPrincipal.getShopName());
        // 首次提问
        if(Objects.isNull(request.getChatCompletionId()) || (Objects.nonNull(ccModel) && StrUtil.isEmpty(ccModel.getFirstQuestion()))) {
            saveModel.setCreateTime(new Date());
            saveModel.setFirstQuestion(request.getContent());
        }
        Integer chatCompletionId = chatCompletionService.saveUserQaSession(saveModel);

        // 保存问答详情
        Integer userQaRecordId = chatCompletionService.saveUserQaRecord(chatCompletionId, 0, null, request.getContent(), request.getAssistantId(), "");

        // 获取随机问题
        final List<HotQaMobileDto> hots = Lists.newArrayList();
//        PageResult<HotQaMobileDto> pageResult = this.hotQaList(new QaAssistantRequestDto());
//        if(Objects.nonNull(pageResult) && CollectionUtil.isNotEmpty(pageResult.getItems())) {
//            List<HotQaMobileDto> items = pageResult.getItems();
//            if(items.size() > 2){
//                Collections.shuffle(items);
//                items = items.subList(0, 2);
//            }
//            hots.addAll(items);
//        }
        List<HotQaResponseModel> hotQaList = chatCompletionService.hotQaList(new HotQaRequestModel());
        if(CollectionUtil.isNotEmpty(hotQaList)) {
            Collections.shuffle(hotQaList);
            if(hotQaList.size() > 5) {
                hotQaList = hotQaList.subList(0, 5);
            }
            hotQaList.forEach(v -> {
                HotQaMobileDto dto = new HotQaMobileDto();
                dto.setQuestion(v.getQuestion());
                hots.add(dto);
            });
        }

        JSONObject reqBody = new JSONObject();
        reqBody.put("content", request.getContent());
        reqBody.put("bot_app_key", dsAppKey);
        reqBody.put("visitor_biz_id", userPrincipal.getUserId());
        reqBody.put("session_id", sessionId);

        QaAssistantResponseModel assistant =chatCompletionService.qaAssistantDetail(request.getAssistantId());
        JSONObject customVariables = new JSONObject();
//        customVariables.put("Position", userPrincipal.getPosition());
//        customVariables.put("Department", userPrincipal.getShopName());
//        customVariables.put("UserId", userPrincipal.getUserId());
        customVariables.put("assistant_name", assistant.getName());
        reqBody.put("custom_variables", customVariables);

        final String[] thoughtContent = {""};
        final String[] replyContent = {""};
        final String[] difySessionId = {""};
        final String[] recordId = {""};
        final Boolean[] replyFlag = {false};
        String finalSessionId = sessionId;
        List<String> references = Lists.newArrayList();
        final Boolean[] referenceFlag = {false};
        return webClient.post().uri(dsApiBase).accept(MediaType.TEXT_EVENT_STREAM).bodyValue(reqBody.toJSONString()).exchangeToFlux(r -> r.bodyToFlux(String.class))
                .mapNotNull(v -> {
                    UserAskResultMobileDto result = new UserAskResultMobileDto();
                    JSONObject json = JSONObject.parseObject(v);
                    String type = json.getString("type");
                    if(Objects.equals(type, "error")) {
                        thoughtContent[0] = json.getJSONObject("error").getString("message");
                    }

                    JSONObject payload = json.getJSONObject("payload");
                    if(Objects.equals(type, "thought")) {
                        JSONArray procedures = payload.getJSONArray("procedures");
                        if(CollectionUtil.isNotEmpty(procedures)) {
                            String content = procedures.getJSONObject(0).getJSONObject("debugging").getString("content");
                            if(StrUtil.isNotEmpty(content)) {
                                thoughtContent[0] = "## 思考中 ##\n";
                                thoughtContent[0] += content;
                            }
                        }
                    }
                    if(Objects.equals(type, "reply") && Objects.equals(payload.getBoolean("is_from_self"), false)) {
                        if(Objects.nonNull(payload)) {
                            String content = payload.getString("content");
                            if(StrUtil.isNotEmpty(content)) {
                                replyContent[0] = "\n\n## 小伊回复 ##\n";
                                replyContent[0] += content;
                                replyFlag[0] = true;
                            }
                            if(ObjectUtil.isEmpty(difySessionId[0]) && ObjectUtil.isNotEmpty(payload.getString("session_id"))) {
                                difySessionId[0] = payload.getString("session_id");
                            }
                            if (StrUtil.isBlank(recordId[0]) && StrUtil.isNotBlank(payload.getString("record_id"))) {
                                recordId[0] = payload.getString("record_id");
                            }
                            if (referenceFlag[0] == false && CollUtil.isEmpty(references) && payload.containsKey("knowledge")) {
                                referenceFlag[0] = true;
                                JSONArray knowledge = payload.getJSONArray("knowledge");
                                List<String> segmentIds = CollUtil.newArrayList();
                                if (ObjectUtil.isNotNull(knowledge) && knowledge.size() > 0) {
                                    for (Object kb : knowledge) {
                                        JSONObject item = (JSONObject) kb;
                                        if (ObjectUtil.equals(2, item.getInteger("type"))) {
                                            segmentIds.add(item.getString("seg_id"));
                                        }
                                    }
                                }
                                if (CollUtil.isNotEmpty(segmentIds)) {
                                    try {
                                        List<String> docs =  tencentCloudFacade.getKbSegment(segmentIds);
                                        if (CollUtil.isNotEmpty(docs)) {
                                            for (String doc : docs) {
                                                references.add(doc);
                                            }
                                        }
                                    } catch (TencentCloudSDKException e) {
                                        log.error("getKbSegment error,{}", e.getMessage());
                                    }
                                }
                            }
                        }
                    }
                    String finalContent = thoughtContent[0];
                    if(replyFlag[0]) {
                        finalContent += replyContent[0];
                    }
                    result.setReferences(references);
                    result.setReplyContent(finalContent);
                    result.setChatCompletionId(chatCompletionId);
                    result.setMessageId(userQaRecordId);
                    result.setHots(hots);
                    return result;
                }).doOnComplete(
                        () -> {
                            String finalContent = thoughtContent[0] + replyContent[0];
                            // 更新DIFY会话ID
                            if(StrUtil.isEmpty(finalSessionId)) {
                                UserChatCompletionSaveModel updateModel = new UserChatCompletionSaveModel();
                                updateModel.setId(chatCompletionId);
                                updateModel.setSessionId(difySessionId[0]);
                                chatCompletionService.saveUserQaSession(updateModel);
                            }
                            // 保存AI问答详情
                            chatCompletionService.saveUserQaRecord(chatCompletionId, 1, userQaRecordId, finalContent, request.getAssistantId(), recordId[0]);
                        }
                ).map(v -> ServerSentEvent.builder(v).build());
    }

    /**
     * 新会话
     * @return
     */
    public NewCompletionResultMobileDto newChatCompletion(UserPrincipal userPrincipal) {
        NewCompletionResultMobileDto result = new NewCompletionResultMobileDto();

        // 新会话信息
        UserChatCompletionSaveModel saveModel = new UserChatCompletionSaveModel();
        saveModel.setUserId(userPrincipal.getUserId());
        saveModel.setUserName(userPrincipal.getUserName());
        saveModel.setShopId(userPrincipal.getShopId());
        saveModel.setShopName(userPrincipal.getShopName());
        saveModel.setCreateTime(new Date());
        Integer recordId = chatCompletionService.saveUserQaSession(saveModel);

        result.setChatCompletionId(recordId);
        return result;
    }


    public AiChatComplateResultDto ask4Knowledge(String sessionId, String userId, String question, Integer assistantId) {
        Map<String, String> inputs = new HashMap<>();
        inputs.put("context", question);
        inputs.put("assistant_id", String.valueOf(assistantId));
        JSONObject param = new JSONObject();
        param.put("inputs", inputs);
        param.put("query", question);
        param.put("response_mode", "blocking");
        param.put("conversation_id", sessionId);
        param.put("user", userId);
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        headers.setBearerAuth(apiKey);

        log.info("异步请求参数1，sessionId = {}", sessionId);
        log.info("异步请求参数2，req = {}", JSONObject.toJSONString(param));
        HttpEntity<String> postEntity = new HttpEntity<>(param.toJSONString(), headers);
        ResponseEntity<String> response = restTemplate.postForEntity(difyBase + "/chat-messages", postEntity, String.class);
        String body = response.getBody();
        log.info("AI问答执行结果：{}", body);
        if(Objects.equals(response.getStatusCode(), HttpStatus.OK)) {
            JSONObject bodyObject = JSONObject.parseObject(body);
            String conversationId = bodyObject.getString("conversation_id");
            String answer = bodyObject.getString("answer");

            AiChatComplateResultDto result = new AiChatComplateResultDto();
            result.setContent(answer);
            result.setDifySessionId(conversationId);
            return result;
        }
        return null;
    }

    /**
     * 热门提问
     * @param request
     * @return
     */
    public PageResult<HotQaMobileDto> hotQaList(QaAssistantRequestDto request) {
        // 获取5个随机自定义热门问题
        final List<HotQaMobileDto> hots = Lists.newArrayList();
        HotQaRequestModel hotQaRequestModel = new HotQaRequestModel();
        hotQaRequestModel.setAssistantId(request.getAssistantId());
        List<HotQaResponseModel> hotQaList = chatCompletionService.hotQaList(hotQaRequestModel);
        if(CollectionUtil.isNotEmpty(hotQaList)) {
            Collections.shuffle(hotQaList);
            if(hotQaList.size() > 5) {
                hotQaList = hotQaList.subList(0, 5);
            }
            hotQaList.forEach(v -> {
                HotQaMobileDto dto = new HotQaMobileDto();
                dto.setQuestion(v.getQuestion());
                hots.add(dto);
            });
        }
        // 获取5个随机系统判断热门问题
        PageResult<HotQaMobileDto> pageResult = kbService.hotQaList(request);
        if(Objects.nonNull(pageResult) && CollectionUtil.isNotEmpty(pageResult.getItems())) {
            List<HotQaMobileDto> items = pageResult.getItems();
            if(items.size() > 5){
                Collections.shuffle(items);
                items = items.subList(0, 5);
            }
            hots.addAll(items);
        }
        pageResult.setItems(hots);
        return pageResult;
    }

    /**
     * 更新 踩 状态
     * @param messageId
     * @param status
     */
    public void updateDissStatus(Integer messageId, Integer status) {
        UserChatCompletionHistoryResponseModel model = chatCompletionService.getReplyMessageByUserMessageId(messageId);
        if(Objects.nonNull(model)) {
            UserQaHistorySaveModel saveModel = new UserQaHistorySaveModel();
            saveModel.setId(model.getId());
            saveModel.setDiss(status);
            chatCompletionService.saveUserQaRecord(saveModel);
            // 提交腾讯云踩的数据
            if (ObjectUtil.equals(1, status)) {
                if (StrUtil.isNotBlank(model.getRecordId())) {
                    try {
                        tencentCloudFacade.dissMessage(dsAppKey, model.getRecordId());
                    } catch (TencentCloudSDKException e) {
                        log.info("提交腾讯云踩数据异常{}", ExceptionUtil.getMessage(e));
                    }
                }
            }
        }
    }

    /**
     * 更新 赞 状态
     * @param messageId
     * @param status
     */
    public void updateLikesStatus(Integer messageId, Integer status) {
        UserChatCompletionHistoryResponseModel model = chatCompletionService.getReplyMessageByUserMessageId(messageId);
        if(Objects.nonNull(model)) {
            UserQaHistorySaveModel saveModel = new UserQaHistorySaveModel();
            saveModel.setId(model.getId());
            saveModel.setLikes(status);
            chatCompletionService.saveUserQaRecord(saveModel);
        }
    }

    /************************************************** 热门问题 **************************************************/

    /**
     * 更新热门问题
     * @param id
     * @param status
     */
    public void updateHotQaStatus(Integer id, Integer status) {
        chatCompletionService.updateHotQaStatus(id, status);
    }

    /**
     * 删除热门问题
     * @param id
     */
    public void deleteHotQa(Integer id) {
        chatCompletionService.deleteHotQa(id);
    }

    /**
     * 获取热门问题详情
     * @param id
     * @return
     */
    public HotQaDto hotQaDetail(Integer id) {
        HotQaResponseModel model = chatCompletionService.hotQaDetail(id);
        HotQaDto result = BeanUtil.copyProperties(model, HotQaDto.class);
        result.setAssistantName("");
        if (ObjectUtil.isNotNull(model.getAssistantId())) {
            QaAssistantResponseModel assistantResponseModel = chatCompletionService.qaAssistantDetail(model.getAssistantId());
            if (ObjectUtil.isNotNull(assistantResponseModel)) {
                result.setAssistantName(assistantResponseModel.getName());
            }
        }
        return result;
    }


    /**
     * 保存热门问题
     * @param dto
     */
    public void saveHotQa(HotQaSaveDto dto) {

        HotQaSaveModel model = BeanUtil.copyProperties(dto, HotQaSaveModel.class);
        chatCompletionService.saveHotQa(model);
    }


    /**
     * 热门问题分页查询
     * @param request
     * @return
     */
    public PageResult<HotQaDto> hotQaPaginQuery(HotQaRequestDto request) {

        HotQaRequestModel requestModel = BeanUtil.copyProperties(request, HotQaRequestModel.class);

        Page<HotQaResponseModel> page = chatCompletionService.hotQaPaginQuery(requestModel);

        PageResult<HotQaDto> pageResult = PageResult.of(request.getPageNo(), request.getPageSize(), (int) page.getTotalRow(), null);

        if(CollectionUtil.isNotEmpty(page.getRecords())) {
            Set<Integer> assistantIds = page.getRecords().stream().map(HotQaResponseModel::getAssistantId).collect(Collectors.toSet());
            final Map<Integer, QaAssistantResponseModel> qaAssistantMap = new HashMap<>();
            if (CollUtil.isNotEmpty(assistantIds)) {
                List<QaAssistantResponseModel> qaAssistantList = chatCompletionService.qaAssistantListByIds(assistantIds);
                Map<Integer, QaAssistantResponseModel> map  = qaAssistantList.stream().collect(Collectors.toMap(QaAssistantResponseModel::getId, v -> v));
                qaAssistantMap.putAll(map);
            }
            List<HotQaDto> dtoList = page.getRecords().stream().map(v -> {
                HotQaDto dto = BeanUtil.copyProperties(v, HotQaDto.class);
                dto.setAssistantName(ObjectUtil.isNotNull(qaAssistantMap.get(v.getAssistantId())) ? qaAssistantMap.get(v.getAssistantId()).getName() : "");
                return dto;
            }).collect(Collectors.toList());
            pageResult.setItems(dtoList);
        }
        return pageResult;
    }

    public List<String> getKbSegment(String segmentId) {
        try {
            return tencentCloudFacade.getKbSegment(List.of(segmentId));
        } catch (TencentCloudSDKException e) {
            throw new RuntimeException(e);
        }
    }
}
